#!/usr/bin/env python
# Copyright 2008-2009 David D Lowe
# released under the GPLv3
"""GTK theme Pigment and PigmentType.
Please don't confuse this module gtk_ with gtk, there are completely different,
hence the underscore"""

from pigment import PigmentType, Pigment
from pigment_more import DirBasedPigment, DirBasedPigmentType
import pigment
import os, sys
import hashlib
import gconf
from glob import glob
import subprocess
import gtk
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import ep_exceptions
import handy
from gettext import gettext as _

# TODO: fix pleaseExit bug

class GTKType(DirBasedPigmentType):
    humanName = {"en":"GTK"}
    humanPluralName = {"en":"GTK"}
    codeName = "gtk"
    abbreviation = "gk"
    directoryName = "gtk"
    canGeneratePreview = True

    attachmentTypeHelp = _("The theme file should be the directory that bears the name of theme, and containing a directory named gtk-2.0, it should not contain any other directories")
    
    def get_Pigment(self):
        return GTK
    
    publicBaseDir = "/usr/share/themes"
    personalBaseDir = "~/.themes"
    
    isDirectoryShared = True
    specificChildren = ("gtk-2.0", "index.theme")

class GTK(DirBasedPigment):
    
    def __init__(self):
        DirBasedPigment.__init__(self)
        self.type = GTKType()
        
    def activate(self, progressCallback=None,shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        client = gconf.client_get_default()
        client.set_string("/desktop/gnome/interface/gtk_theme", self.attDirname)
        progressCallback(1.0)

    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if DirBasedPigment.is_installed(self, installationDirectory):
            if os.path.exists(os.path.join(self.type.publicBaseDir,self.attDirname, "gtk-2.0")):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()

    def is_attachment_correct(self, attachmentIndex=None):
        if not DirBasedPigment.is_attachment_correct(self, attachmentIndex):
            return False
        if attachmentIndex is None:
            att = self.attachments[0]
        else:
            att = self.attachments[attachmentIndex]
        if att[-1] == "/": att = att[:-1]
        dir = att.split("/")[-1]
        if dir in ("metacity-1", "gtk-2.0"):
            return False
        if not os.path.isdir(os.path.join(att,"gtk-2.0")):
            return False
        if self.get_state() != pigment.STATE_INSTALLED:
            for dir in os.listdir(att):
                if os.path.isdir(dir):
                    if dir != "gtk-2.0":
                        return False
        if len(os.listdir(os.path.join(att, "gtk-2.0"))) == 0:
            return False
        return True
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = glob(self.attPath + "/gtk-2.0/*") # = file(self.attPath, 'rb')
            f2 = glob("/usr/share/themes/%s/gtk-2.0/*" % self.attDirname) #file(os.path.join("/usr/share/", self.codeName, self.wallpaperFilename), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        client = gconf.client_get_default()
        try:
            f1 = glob(self.attPath + "/gtk-2.0/*")
            f2 = glob("/usr/share/themes/%s/gtk-2.0/*" % client.get_string("/desktop/gnome/interface/gtk_theme")) 
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def generate_preview(self):
        if len(self.attPath) < 1:
            return None
        dir = os.path.dirname(os.path.abspath(__file__))
        themeScript = os.path.join(dir,  "theme_preview.py")
        subprocess.call(["mkdir",  "-p",  "/tmp/epidermis"])
        subprocess.call(["rm","-f",  "/tmp/epidermis/gtk-preview.png"])
        env = os.environ
        env["GTK2_RC_FILES"] = os.path.join(self.attPath,  "gtk-2.0",  "gtkrc")
        subprocess.call(["python", themeScript, "gtk",  "/tmp/epidermis/gtk-preview.png"], env=env)
        try:
            pb = gtk.gdk.pixbuf_new_from_file("/tmp/epidermis/gtk-preview.png")
        except ee:
            import traceback
            traceback.print_exc()
            return None
        if not pb is None:
            self.preview = pb
            return pb
        else:
            return None  
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = GTK()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()
